           (*-------------------------------------------*
            |           November 2007 for Isabelle 2005 |
            |           November 2008 for Isabelle 2008 |
            |                                           |
            |        Yoshinao Isobe (AIST JAPAN)        |
            *-------------------------------------------*)

theory CSP_F_op_pipe_v4_1
imports CSP_F_domain CSP_syntax_v4_1 Trace_v4_1 CSP_F_op_alpha_par
begin

(*****************************************************************

         1. semantic clauses for Pipe

 *****************************************************************)

(* -------------------------------------------------------------- *
                 PIPE  (P <[left,mid,right]> Q)
 * -------------------------------------------------------------- *)

consts 
  pipe_condition_prefix :: 
     "'a trace => 'a trace => 'a trace => 
       ('x => 'a) => ('x => 'a) => bool"

  pipe_condition_heads :: 
     "'a trace => 'a trace => 'a trace => ('x => 'a) => ('x => 'a) => bool"

defs
  pipe_condition_prefix_def:
    "pipe_condition_prefix u s t left right ==
      (ALL u'. prefix u' u -->
        (EX s'. prefix s' s &
           (EX t'. prefix t' t &
             u' rest-tr range left = s' rest-tr range left &
            (s' rest-tr range right) [[right<==>left]]* (t' rest-tr range left) &
             t' rest-tr range right = u' rest-tr range right)))"

  pipe_condition_heads_def:
    "pipe_condition_heads u s t left right ==
       (u=<> | s=<> | t=<> | (hdt u = hdt s) | (hdt u = hdt t) | 
        ((inv Ev (hdt s), inv Ev (hdt t)) : right<==>left))"

(* --------------------------------- *
     a condition for head of traces 
 * --------------------------------- *)

lemma prefix_pipe_condition_heads:
  "ALL left right u s t.
     (inj left & inj right &
      (ALL x y. right x ~= left y) &
      noTick s & noTick t & noTick u &
      sett s <= insert Tick (Ev ` (range left Un range right)) &
      sett t <= insert Tick (Ev ` (range left Un range right)) &
      sett u <= insert Tick (Ev ` (range left Un range right)) &
      pipe_condition_prefix u s t left right)
    --> pipe_condition_heads u s t left right"
apply (intro allI impI)
apply (elim conjE exE)
apply (simp add: pipe_condition_heads_def)
apply (simp add: pipe_condition_prefix_def)
apply (insert trace_nil_or_Tick_or_Ev)
apply (rotate_tac -1)
apply (frule_tac x="u" in spec)
apply (frule_tac x="s" in spec)
apply (drule_tac x="t" in spec)
apply (elim disjE)
apply (simp_all)
apply (elim conjE exE)
apply (simp add: image_iff)
apply (drule_tac x="<Ev a>" in spec)
apply (elim add_not_eq_symE)
apply (simp add: image_iff)
apply (elim disjE conjE exE)
apply (simp_all)
apply (simp add: image_iff)
apply (simp add: image_iff)
done

(* --- corollary --- *)

lemma prefix_pipe_condition_heads_false1:
  "[| inj left ; inj right ;
      ALL x y. right x ~= left y ;
      noTick s ; noTick t ; noTick u ;
      sett s <= insert Tick (Ev ` (range left Un range right)) ;
      sett t <= insert Tick (Ev ` (range left Un range right)) ;
      sett u <= insert Tick (Ev ` (range left Un range right)) |]
    ==> ~ pipe_condition_prefix
            (<Ev (right z)> ^^ u)
            (<Ev (left x)> ^^ s)
            (<Ev (left y)> ^^ t) left right"
apply (insert prefix_pipe_condition_heads)
apply (drule_tac x="left" in spec)
apply (drule_tac x="right" in spec)
apply (drule_tac x="<Ev (right z)> ^^ u" in spec)
apply (drule_tac x="<Ev (left x)> ^^ s" in spec)
apply (drule_tac x="<Ev (left y)> ^^ t" in spec)
apply (simp add: image_iff)
apply (elim add_not_eq_symE)
apply (simp)
apply (rotate_tac 3)
apply (erule contrapos_pn)
apply (simp)
apply (simp add: pipe_condition_heads_def)
apply (simp add: pair_in_Renaming_channel)
apply (auto)
done

lemma prefix_pipe_condition_heads_false2:
  "[| inj left ; inj right ;
      ALL x y. right x ~= left y ;
      noTick s ; noTick t ; noTick u ;
      sett s <= insert Tick (Ev ` (range left Un range right)) ;
      sett t <= insert Tick (Ev ` (range left Un range right)) ;
      sett u <= insert Tick (Ev ` (range left Un range right)) |]
    ==> ~ pipe_condition_prefix
            (<Ev (left z)> ^^ u)
            (<Ev (right x)> ^^ s)
            (<Ev (right y)> ^^ t) left right"
apply (insert prefix_pipe_condition_heads)
apply (drule_tac x="left" in spec)
apply (drule_tac x="right" in spec)
apply (drule_tac x="<Ev (left z)> ^^ u" in spec)
apply (drule_tac x="<Ev (right x)> ^^ s" in spec)
apply (drule_tac x="<Ev (right y)> ^^ t" in spec)
apply (simp add: image_iff)
apply (elim add_not_eq_symE)
apply (simp)
apply (rotate_tac 3)
apply (erule contrapos_pn)
apply (simp)
apply (simp add: pipe_condition_heads_def)
apply (simp add: pair_in_Renaming_channel)
apply (auto)
done

lemmas prefix_pipe_condition_heads_false =
       prefix_pipe_condition_heads_false1
       prefix_pipe_condition_heads_false2

(* -------------------------------------------- *
          pipe_condition (delet heads)
 * -------------------------------------------- *)

lemma pipe_condition_prefix_del1:
  "[| ALL x y. right x ~= left y ;
      pipe_condition_prefix 
          u (<Ev (right x)>^^s) (<Ev (left x)> ^^ t) left right |]
  ==> pipe_condition_prefix u s t left right"
apply (simp add: pipe_condition_prefix_def)
apply (intro allI impI)
apply (drule_tac x="u'" in spec)
apply (simp)
apply (elim disjE conjE exE)
apply (simp)
apply (rule_tac x="<>" in exI, simp)
apply (rule_tac x="<>" in exI, simp)
apply (rule_tac x="<>" in exI, simp)
apply (rule_tac x="v'" in exI, simp)
apply (rule_tac x="v'" in exI, simp)
apply (rule_tac x="v'a" in exI, simp)
apply (elim add_not_eq_symE)
apply (simp add: image_iff)
done

lemma pipe_condition_prefix_del2:
  "[| ALL x y. right x ~= left y ;
      pipe_condition_prefix 
          (<Ev (right x)>^^u) s (<Ev (right x)> ^^ t) left right |]
  ==> pipe_condition_prefix u s t left right"
apply (simp add: pipe_condition_prefix_def)
apply (intro allI impI)
apply (drule_tac x="<Ev (right x)> ^^ u'" in spec)
apply (simp)
apply (elim disjE conjE exE)
apply (simp)
apply (simp)
apply (rule_tac x="s'" in exI, simp)
apply (rule_tac x="v'" in exI, simp)
apply (elim add_not_eq_symE)
apply (simp add: image_iff)
done

lemma pipe_condition_prefix_del3:
  "[| ALL x y. right x ~= left y ;
      pipe_condition_prefix 
          (<Ev (left x)>^^u) (<Ev (left x)> ^^ s) t left right |]
  ==> pipe_condition_prefix u s t left right"
apply (simp add: pipe_condition_prefix_def)
apply (intro allI impI)
apply (drule_tac x="<Ev (left x)> ^^ u'" in spec)
apply (simp)
apply (elim disjE conjE exE)
apply (simp)
apply (simp)
apply (rule_tac x="v'" in exI, simp)
apply (rule_tac x="t'" in exI, simp)
apply (elim add_not_eq_symE)
apply (simp add: image_iff)
done

(* -------------------------------------------- *
            pipe_condition (add heads)
 * -------------------------------------------- *)

lemma pipe_condition_prefix_add1:
  "[| inj left ; inj right ; ALL x y. right x ~= left y ;
      pipe_condition_prefix u s t left right |]
  ==> pipe_condition_prefix u (<Ev (right x)> ^^ s) (<Ev (left x)> ^^ t) left right"
apply (simp add: pipe_condition_prefix_def)
apply (intro conjI allI impI)

apply (drule_tac x="u'" in spec)
apply (simp)
apply (elim conjE exE)
apply (rule_tac x="<Ev (right x)> ^^ s'" in exI, simp)
apply (rule_tac x="<Ev (left x)> ^^ t'" in exI, simp)
apply (elim add_not_eq_symE)
apply (simp add: image_iff)
apply (simp add: pair_in_Renaming_channel)
done

lemma pipe_condition_prefix_add2:
  "[| inj left ; inj right ; ALL x y. right x ~= left y ;
      pipe_condition_prefix u s t left right |]
  ==> pipe_condition_prefix (<Ev (right x)> ^^ u) s (<Ev (right x)> ^^ t) left right"
apply (simp add: pipe_condition_prefix_def)
apply (intro conjI allI impI)

(* 1 *)
 apply (drule_tac x="<>" in spec)
 apply (simp)
 apply (elim disjE conjE exE)
  apply (rule_tac x="<>" in exI)
  apply (simp)
  apply (rule_tac x="<>" in exI)
  apply (simp)

(* 2 *)
 apply (elim disjE conjE exE)
 apply (drule_tac x="v'" in spec)
 apply (simp)
 apply (elim disjE conjE exE)
 apply (simp)
 apply (rule_tac x="s'" in exI)
 apply (simp)
 apply (rule_tac x="<Ev (right x)> ^^ t'" in exI)
 apply (simp)
 apply (simp add: image_iff)
done

lemma pipe_condition_prefix_add3:
  "[| inj left ; inj right ; ALL x y. right x ~= left y ;
      pipe_condition_prefix u s t left right |]
  ==> pipe_condition_prefix (<Ev (left x)> ^^ u) (<Ev (left x)> ^^ s) t left right"
apply (simp add: pipe_condition_prefix_def)
apply (intro conjI allI impI)

(* 1 *)
 apply (drule_tac x="<>" in spec)
 apply (simp)
 apply (elim disjE conjE exE)
  apply (rule_tac x="<>" in exI)
  apply (simp)
  apply (rule_tac x="<>" in exI)
  apply (simp)

(* 2 *)
 apply (elim disjE conjE exE)
 apply (drule_tac x="v'" in spec)
 apply (simp)
 apply (elim disjE conjE exE)
 apply (simp)
 apply (rule_tac x="<Ev (left x)> ^^ s'" in exI)
 apply (simp)
 apply (rule_tac x="t'" in exI)
 apply (simp)
 apply (elim add_not_eq_symE)
 apply (simp add: image_iff)
done

(* -------------------------------------------- *
            pipe_condition (heads)
 * -------------------------------------------- *)

lemma pipe_condition_prefix_head1:
  "[| inj left ; inj right ; ALL x y. right x ~= left y |]
  ==>
   pipe_condition_prefix u (<Ev (right x)>^^s) (<Ev (left x)> ^^ t) left right
   = pipe_condition_prefix u s t left right"
apply (rule)
apply (rule pipe_condition_prefix_del1)
apply (simp_all)
apply (simp add: pipe_condition_prefix_add1)
done

lemma pipe_condition_prefix_head2:
  "[| inj left ; inj right ; ALL x y. right x ~= left y |]
  ==>
   pipe_condition_prefix (<Ev (right x)>^^u) s (<Ev (right x)> ^^ t) left right
   = pipe_condition_prefix u s t left right"
apply (rule)
apply (rule pipe_condition_prefix_del2)
apply (simp_all)
apply (simp add: pipe_condition_prefix_add2)
done

lemma pipe_condition_prefix_head3:
  "[| inj left ; inj right ; ALL x y. right x ~= left y |]
  ==>
   pipe_condition_prefix (<Ev (left x)>^^u) (<Ev (left x)> ^^ s) t left right
   = pipe_condition_prefix u s t left right"
apply (rule)
apply (rule pipe_condition_prefix_del3)
apply (simp_all)
apply (simp add: pipe_condition_prefix_add3)
done

lemmas pipe_condition_prefix_head =
       pipe_condition_prefix_head1
       pipe_condition_prefix_head2
       pipe_condition_prefix_head3

(* -------------------------------------------- *
                 remove Tick 
 * -------------------------------------------- *)

lemma pipe_condition_prefix_rem_Tick:
  "ALL u s t.
   (noTick u & noTick s & noTick t & 
    pipe_condition_prefix (u ^^ <Tick>) (s ^^ <Tick>) (t ^^ <Tick>) left right)
    --> pipe_condition_prefix u s t left right"
apply (simp add: pipe_condition_prefix_def)
apply (intro allI impI)
apply (elim conjE exE)
apply (drule_tac x="u'" in spec)
apply (simp)
apply (elim conjE exE)
apply (elim disjE)
apply (simp_all)

 apply (subgoal_tac "noTick(s rest-tr range left ^^ <Tick>)")
 apply (simp)
 apply (rotate_tac 4)
 apply (drule sym)
 apply (simp add: prefix_noTick)

 apply (subgoal_tac "noTick(s rest-tr range left ^^ <Tick>)")
 apply (simp)
 apply (rotate_tac -5)
 apply (drule sym)
 apply (simp add: prefix_noTick)

 apply (rotate_tac -3)
 apply (drule sym)
 apply (subgoal_tac "noTick(t rest-tr range right ^^ <Tick>)")
 apply (simp)
 apply (rotate_tac -1)
 apply (drule sym)
 apply (simp add: prefix_noTick)

apply (auto)
done

lemma pipe_condition_prefix_add_Tick:
  "ALL u s t.
   (noTick u & noTick s & noTick t & 
         u rest-tr range left = s rest-tr range left &
        (s rest-tr range right) [[right<==>left]]* (t rest-tr range left) &
         t rest-tr range right = u rest-tr range right &
    pipe_condition_prefix u s t left right)
    --> pipe_condition_prefix (u ^^ <Tick>) (s ^^ <Tick>) (t ^^ <Tick>) left right"
apply (simp add: pipe_condition_prefix_def)
apply (intro allI impI)
apply (elim conjE exE)
apply (rule)

(* Tick *)
 apply (intro allI impI)
 apply (rule_tac x="s ^^ <Tick>" in exI, simp)
 apply (rule_tac x="t ^^ <Tick>" in exI, simp)

(* noTick *)
 apply (intro allI impI)
 apply (drule_tac x="u'" in spec)
 apply (simp)
 apply (elim conjE exE)
 apply (rule_tac x="s'" in exI, simp)
 apply (rule_tac x="t'" in exI, simp)
done

(* ----------- *
    simp Tick 
 * ----------- *)

lemma pipe_condition_prefix_simp_Tick:
  "[| noTick u ; noTick s ; noTick t ; 
         u rest-tr range left = s rest-tr range left ;
        (s rest-tr range right) [[right<==>left]]* (t rest-tr range left) ;
         t rest-tr range right = u rest-tr range right |]
   ==> pipe_condition_prefix (u ^^ <Tick>) (s ^^ <Tick>) (t ^^ <Tick>) left right =
       pipe_condition_prefix u s t left right"
apply (rule)
apply (insert pipe_condition_prefix_rem_Tick[of left right])
apply (drule_tac x="u" in spec)
apply (drule_tac x="s" in spec)
apply (drule_tac x="t" in spec)
apply (simp)
apply (rotate_tac -1)
apply (erule rem_asmE)
apply (simp add: pipe_condition_prefix_add_Tick)
done

(* ------------------------------------------------------ *
                   traces composition
 * ------------------------------------------------------ *)

(* -------------------------------------------- *
  pipe_condition --> a composed trace v (noTick)
 * -------------------------------------------- *)

lemma pipe_condition_prefix_comp_noTick:
 "ALL n u s t.
  (inj left & inj mid & inj right &
   (ALL x y. right x ~= mid y) & 
   (ALL x y. left x ~= mid y) & 
   (ALL x y. right x ~= left y) &
   noTick s & noTick t & noTick u &
   lengtht u + lengtht s + lengtht t <= n &
   sett s <= insert Tick (Ev ` (range left Un range right)) &
   sett t <= insert Tick (Ev ` (range left Un range right)) &
   sett u <= insert Tick (Ev ` (range left Un range right)) &
             u rest-tr range left = s rest-tr range left &
            (s rest-tr range right) [[right<==>left]]* (t rest-tr range left) &
             t rest-tr range right = u rest-tr range right &
    pipe_condition_prefix u s t left right)
   --> (EX v. u = v --tr (range mid) &
              s [[right<==>mid]]* (v rest-tr (range left Un range mid)) &
               t [[left<==>mid]]* (v rest-tr (range mid Un range right)) &
        sett v <= insert Tick 
                  (Ev ` (range left Un range mid Un (range mid Un range right))))"
apply (rule)
apply (induct_tac n)
 apply (simp add: lengtht_zero)
 apply (intro allI impI)
 apply (rule_tac x="<>" in exI)
 apply (simp)

 (* Suc n *)
 apply (intro allI impI)
 apply (elim conjE exE)
 apply (insert trace_nil_or_Tick_or_Ev)
 apply (rotate_tac -1)
 apply (frule_tac x="u" in spec)
 apply (frule_tac x="s" in spec)
 apply (drule_tac x="t" in spec)
 apply (elim disjE conjE exE)
 apply (simp_all)

 (* 0 *)
 apply (rule_tac x="<>" in exI)
 apply (simp)

 (* 1 *)
  apply (elim conjE exE)
  apply (drule_tac x="<>" in spec)
  apply (drule_tac x="<>" in spec)
  apply (drule_tac x="sa" in spec)
  apply (simp)
  apply (drule mp)
   apply (simp add: pipe_condition_prefix_def)
   apply (rule_tac x="<>" in exI)
   apply (simp)
  apply (simp add: image_iff)
  apply (elim disjE conjE exE)
  apply (simp)
  apply (simp)

 (* 2 *)
  apply (drule_tac x="<>" in spec)
  apply (drule_tac x="sa" in spec)
  apply (drule_tac x="<>" in spec)
  apply (simp)
  apply (drule mp)
   apply (simp add: pipe_condition_prefix_def)
   apply (rule_tac x="<>" in exI)
   apply (simp)
  apply (simp add: image_iff)
  apply (elim disjE conjE exE)
  apply (simp)
  apply (simp)

 (* 3 *)
  apply (drule_tac x="<>" in spec)
  apply (drule_tac x="sa" in spec)
  apply (drule_tac x="sb" in spec)
  apply (simp)
  apply (elim conjE exE)
  apply (simp add: image_iff)
  apply (elim disjE)
   (* i *)
   apply (elim conjE exE)
   apply (simp)
   (* ii *)
   apply (elim conjE exE)
   apply (simp)
   (* iii *)
   apply (elim conjE exE)
   apply (simp)
   apply (simp add: image_iff)
   apply (simp add: pair_in_Renaming_channel)
   apply (simp add: pipe_condition_prefix_head)
   apply (elim conjE exE)
   apply (rule_tac x="<Ev (mid xa)> ^^ v" in exI)
   apply (simp)
   apply (simp add: pair_in_Renaming_channel)

   (* iv *)
   apply (elim conjE exE)
   apply (simp)

 (* 4 *)
  apply (drule_tac x="sa" in spec)
  apply (drule_tac x="<>" in spec)
  apply (drule_tac x="<>" in spec)
  apply (simp)
  apply (elim conjE exE)
  apply (simp add: image_iff)
  apply (elim disjE)
   (* i *)
   apply (elim conjE exE)
   apply (simp)
   (* ii *)
   apply (elim conjE exE)
   apply (simp)

 (* 5 *)
  apply (drule_tac x="sa" in spec)
  apply (drule_tac x="<>" in spec)
  apply (drule_tac x="sb" in spec)
  apply (simp)
  apply (elim conjE exE)
  apply (simp add: image_iff)
  apply (elim disjE)
   (* i *)
   apply (elim conjE exE)
   apply (simp)
   (* ii *)
   apply (elim conjE exE)
   apply (simp)
   (* iii *)
   apply (elim conjE exE)
   apply (simp)
   (* iv *)
   apply (elim conjE exE)
   apply (simp)
   apply (simp add: pipe_condition_prefix_head)
   apply (elim conjE exE)
   apply (rule_tac x="<Ev (right xa)> ^^ v" in exI)
   apply (simp)
   apply (simp add: image_iff)
   apply (simp add: pair_in_Renaming_channel)

 (* 6 *)
  apply (drule_tac x="sa" in spec)
  apply (drule_tac x="sb" in spec)
  apply (drule_tac x="<>" in spec)
  apply (simp)
  apply (elim conjE exE)
  apply (simp add: image_iff)
  apply (elim disjE)
   (* i *)
   apply (elim conjE exE)
   apply (simp)
   apply (simp add: pipe_condition_prefix_head)
   apply (elim conjE exE)
   apply (rule_tac x="<Ev (left x)> ^^ v" in exI)
   apply (simp)
   apply (simp add: image_iff)
   apply (elim add_not_eq_symE)
   apply (simp add: pair_in_Renaming_channel)
   apply (subgoal_tac "left x ~: range mid Un range right", simp)
   apply (force)
   (* ii *)
   apply (elim conjE exE)
   apply (simp)
   (* iii *)
   apply (elim conjE exE)
   apply (simp)
   (* iv *)
   apply (elim conjE exE)
   apply (simp)

 (* 7 *)
  apply (simp add: image_iff)
  apply (elim add_not_eq_symE)
  apply (elim disjE conjE exE)
  apply (simp_all)

 (* 7.1 *)
  apply (drule_tac x="sa" in spec)
  apply (drule_tac x="sb" in spec)
  apply (drule_tac x="<Ev (left xa)> ^^ sc" in spec)
  apply (simp)
  apply (simp add: pipe_condition_prefix_head)
  apply (simp add: image_iff)
  apply (elim conjE exE)
  apply (simp)
  apply (rule_tac x="<Ev (left x)> ^^ v" in exI)
  apply (simp add: image_iff)
  apply (simp add: pair_in_Renaming_channel)
  apply (force)

 (* 7.2 *)
  apply (simp add: prefix_pipe_condition_heads_false)

 (* 7.3 *)
  apply (drule_tac x="sa" in spec)
  apply (drule_tac x="sb" in spec)
  apply (drule_tac x="<Ev (right xa)> ^^ sc" in spec)
  apply (simp)
  apply (simp add: pipe_condition_prefix_head)
  apply (simp add: image_iff)
  apply (elim conjE exE)

  apply (simp)
  apply (rule_tac x="<Ev (left x)> ^^ v" in exI)
  apply (simp add: image_iff)
  apply (simp add: pair_in_Renaming_channel)
  apply (force)

 (* 7.4 *)
  apply (drule_tac x="sa" in spec)
  apply (drule_tac x="<Ev (left x)> ^^ sb" in spec)
  apply (drule_tac x="sc" in spec)
  apply (simp)
  apply (simp add: pipe_condition_prefix_head)
  apply (simp add: image_iff)
  apply (elim conjE exE)

  apply (simp)
  apply (rule_tac x="<Ev (right xb)> ^^ v" in exI)
  apply (simp add: image_iff)
  apply (simp add: pair_in_Renaming_channel)
  apply (force)

 (* 7.5 *)
  apply (drule_tac x="<Ev (left xb)> ^^ sa" in spec)
  apply (drule_tac x="sb" in spec)
  apply (drule_tac x="sc" in spec)
  apply (simp)
  apply (simp add: pair_in_Renaming_channel)
  apply (simp add: pipe_condition_prefix_head)
  apply (simp add: image_iff)
  apply (elim conjE exE)

  apply (simp)
  apply (rule_tac x="<Ev (mid xa)> ^^ v" in exI)
  apply (simp add: image_iff)
  apply (simp add: pair_in_Renaming_channel)
  apply (force)

 (* 7.6 *)
  apply (drule_tac x="<Ev (right xb)> ^^ sa" in spec)
  apply (drule_tac x="sb" in spec)
  apply (drule_tac x="sc" in spec)
  apply (simp)
  apply (simp add: pair_in_Renaming_channel)
  apply (simp add: pipe_condition_prefix_head)
  apply (simp add: image_iff)
  apply (elim conjE exE)

  apply (simp)
  apply (rule_tac x="<Ev (mid xa)> ^^ v" in exI)
  apply (simp add: image_iff)
  apply (simp add: pair_in_Renaming_channel)
  apply (force)

 (* 7.7 *)
  apply (simp add: prefix_pipe_condition_heads_false)

 (* 7.8 *)
  apply (drule_tac x="sa" in spec)
  apply (drule_tac x="<Ev (right x)> ^^ sb" in spec)
  apply (drule_tac x="sc" in spec)
  apply (simp)
  apply (simp add: pipe_condition_prefix_head)
  apply (simp add: image_iff)
  apply (elim conjE exE)

  apply (rule_tac x="<Ev (right xb)> ^^ v" in exI)
  apply (simp add: image_iff)
  apply (simp add: pair_in_Renaming_channel)
  apply (force)
done

(* -------------------------------------------- *
       pipe_condition --> noTick or Tick
 * -------------------------------------------- *)

lemma pipe_condition_noTick_or_Tick:
     "[| u rest-tr range left = s rest-tr range left ;
         t rest-tr range right = u rest-tr range right |]
      ==> (noTick s & noTick t & noTick u | 
         ~ noTick s & ~ noTick t & ~ noTick u)"
 apply (case_tac "noTick u")
 apply (simp)
 apply (subgoal_tac "noTick (u rest-tr range left)")
 apply (simp)
 apply (subgoal_tac "noTick (u rest-tr range right)")
 apply (rotate_tac 1)
 apply (drule sym)
 apply (simp)
 apply (simp)
 apply (drule sym)
 apply (simp)

 apply (simp)
 apply (subgoal_tac "~ noTick (u rest-tr range left)")
 apply (simp)
 apply (subgoal_tac "~ noTick (u rest-tr range right)")
 apply (rotate_tac 1)
 apply (drule sym)
 apply (simp)
 apply (simp)
 apply (drule sym)
 apply (simp)
done

(* -------------------------------------------- *
   pipe_condition --> a composed trace v (Tick)
 * -------------------------------------------- *)

lemma pipe_condition_prefix_comp_Tick:
 "ALL u s t.
  (inj left & inj mid & inj right &
   (ALL x y. right x ~= mid y) & 
   (ALL x y. left x ~= mid y) & 
   (ALL x y. right x ~= left y) &
   ~ noTick s & ~ noTick t & ~ noTick u &
   sett s <= insert Tick (Ev ` (range left Un range right)) &
   sett t <= insert Tick (Ev ` (range left Un range right)) &
   sett u <= insert Tick (Ev ` (range left Un range right)) &
             u rest-tr range left = s rest-tr range left &
            (s rest-tr range right) [[right<==>left]]* (t rest-tr range left) &
             t rest-tr range right = u rest-tr range right &
    pipe_condition_prefix u s t left right)
   --> (EX v. u = v --tr (range mid) &
              s [[right<==>mid]]* (v rest-tr (range left Un range mid)) &
               t [[left<==>mid]]* (v rest-tr (range mid Un range right)) &
        sett v <= insert Tick 
                  (Ev ` (range left Un range mid Un (range mid Un range right))))"
apply (simp add: not_noTick_in_sett)
apply (intro allI impI)
apply (elim conjE exE)
apply (rename_tac u s t s' t' u')
apply (simp)
apply (simp add: pipe_condition_prefix_simp_Tick)
apply (insert pipe_condition_prefix_comp_noTick[of left mid right])
apply (drule_tac x="lengtht u' + lengtht s' + lengtht t'" in spec)
apply (drule_tac x="u'" in spec)
apply (drule_tac x="s'" in spec)
apply (drule_tac x="t'" in spec)
apply (simp)
apply (elim conjE exE)
apply (rule_tac x="v ^^ <Tick>" in exI)
apply (simp add: pipe_condition_prefix_simp_Tick)
done

(* -------------------------------------------- *
      pipe_condition --> a composed trace v
 * -------------------------------------------- *)

lemma pipe_condition_prefix_comp:
 "ALL u s t.
  (inj left & inj mid & inj right &
   (ALL x y. right x ~= mid y) & 
   (ALL x y. left x ~= mid y) & 
   (ALL x y. right x ~= left y) &
   sett s <= insert Tick (Ev ` (range left Un range right)) &
   sett t <= insert Tick (Ev ` (range left Un range right)) &
   sett u <= insert Tick (Ev ` (range left Un range right)) &
             u rest-tr range left = s rest-tr range left &
            (s rest-tr range right) [[right<==>left]]* (t rest-tr range left) &
             t rest-tr range right = u rest-tr range right &
    pipe_condition_prefix u s t left right)
   --> (EX v. u = v --tr (range mid) &
              s [[right<==>mid]]* (v rest-tr (range left Un range mid)) &
               t [[left<==>mid]]* (v rest-tr (range mid Un range right)) &
        sett v <= insert Tick 
                  (Ev ` (range left Un range mid Un (range mid Un range right))))"
apply (intro allI impI)
apply (elim conjE exE)
apply (subgoal_tac "(noTick s & noTick t & noTick u | 
                   ~ noTick s & ~ noTick t & ~ noTick u)")
apply (erule disjE)
apply (insert pipe_condition_prefix_comp_noTick[of left mid right])
 apply (drule_tac x="lengtht u + lengtht s + lengtht t" in spec)
 apply (drule_tac x="u" in spec)
 apply (drule_tac x="s" in spec)
 apply (drule_tac x="t" in spec)
apply (simp)
apply (simp add: pipe_condition_prefix_comp_Tick)
apply (simp add: pipe_condition_noTick_or_Tick)
done

(* ====================== traces ====================== *)

lemma in_traces_Pipe_comp_sub:
  "[| inj left; inj mid; inj right; ALL x y. right x ~= mid y; ALL x y. left x ~= mid y;
          ALL x y. right x ~= left y; u = v --tr range mid;
          sett v <= insert Tick (Ev ` (range left Un range mid Un (range mid Un range right)));
          s [[right<==>mid]]* (v rest-tr (range left Un range mid));
          t [[left<==>mid]]* (v rest-tr (range mid Un range right)) |]
       ==> sett s <= insert Tick (Ev ` (range left Un range right)) &
           sett t <= insert Tick (Ev ` (range left Un range right)) &
           sett (v --tr range mid) <= insert Tick (Ev ` (range left Un range right)) &
           v --tr range mid rest-tr range left = s rest-tr range left &
           (s rest-tr range right) [[right<==>left]]* (t rest-tr range left) &
           t rest-tr range right = v --tr range mid rest-tr range right &
           pipe_condition_prefix (v --tr range mid) s t left right"
 apply (elim add_not_eq_symE)
 apply (intro conjI)

 (* 1 *)
 apply (subgoal_tac  
   "sett (v rest-tr (range left Un range mid)) <= 
    insert Tick (Ev ` (range left Un range mid))")
 apply (rule Renaming_channel_sett4)
 apply (rotate_tac 1)
 apply (simp)
 apply (simp)
 apply (simp)
 apply (simp)
 apply (simp)
 apply (simp)
 apply (assumption)
 apply (simp)
 apply (simp)

 (* 2 *)
 apply (subgoal_tac  
   "sett (v rest-tr (range mid Un range right)) <=
         insert Tick (Ev ` (range mid Un range right))")
 apply (rule Renaming_channel_sett3)
 apply (rotate_tac 1)
 apply (simp)
 apply (simp)
 apply (simp)
 apply (simp)
 apply (simp)
 apply (simp)
 apply (assumption)
 apply (simp)
 apply (simp)

 (* 3 *)
 apply (rule)
 apply (erule subsetE)
 apply (drule_tac x="x" in bspec, simp)
 apply (simp)
 apply (fast)

 (* 4 *)
 apply (subgoal_tac "s rest-tr range left = v rest-tr range left")
 apply (simp)
 apply (rule hide_tr_of_rest_tr_empty1)
 apply (fast)
 apply (simp add: Renaming_channel_tr_rest_eq_range)
 apply (simp add: Un_upper1 Un_upper2 rest_tr_of_rest_tr_subset2)

 (* 5 *)
 apply (rule ren_tr_rest_Renaming_channel)
 apply (simp)
 apply (rotate_tac 1)
 apply (rule)
 apply (simp)+
 apply (rule)
 apply (simp_all)

 (* 6 *)
 apply (subgoal_tac "t rest-tr range right = v rest-tr range right")
 apply (simp)
 apply (rule sym)
 apply (rule hide_tr_of_rest_tr_empty1)
 apply (fast)
 apply (simp add: Renaming_channel_tr_rest_eq_range)
 apply (simp add: Un_upper1 Un_upper2 rest_tr_of_rest_tr_subset2)

 (* 7 *)
  apply (simp add: pipe_condition_prefix_def)
  apply (intro allI impI)
  apply (simp add: hide_tr_prefix)
  apply (elim conjE exE)
  apply (rename_tac u' v')
  apply (simp)
  apply (subgoal_tac "prefix (v' rest-tr (range left Un range mid))
                              (v rest-tr (range left Un range mid))")
  apply (rotate_tac -1)
  apply (erule ren_tr_prefixE)
  apply (simp)
  apply (subgoal_tac "prefix (v' rest-tr (range mid Un range right))
                              (v rest-tr (range mid Un range right))")
  apply (rotate_tac -1)
  apply (erule ren_tr_prefixE)
  apply (simp)
  apply (rename_tac u' v' s' t')
  apply (rule_tac x="s'" in exI, simp)
  apply (rule_tac x="t'" in exI, simp)

  apply (rule)
   apply (subgoal_tac "s' rest-tr range left = 
         (v' rest-tr (range left Un range mid)) rest-tr range left")
   apply (simp add: Un_upper1 Un_upper2 rest_tr_of_rest_tr_subset2)
   apply (rule hide_tr_of_rest_tr_empty1)
   apply (fast)
   apply (simp add: Renaming_channel_tr_rest_eq_range)

  apply (rule)
   apply (rule ren_tr_rest_Renaming_channel)
   apply (simp)
   apply (rotate_tac 1)
   apply (rule)
   apply (simp)+
   apply (rule)
   apply (simp_all)

   apply (subgoal_tac "t' rest-tr range right = 
         (v' rest-tr (range mid Un range right)) rest-tr range right")
   apply (simp add: Un_upper1 Un_upper2 rest_tr_of_rest_tr_subset2)
   apply (rule sym)
   apply (rule hide_tr_of_rest_tr_empty1)
   apply (fast)
   apply (simp add: Renaming_channel_tr_rest_eq_range)

   apply (simp add: rest_tr_def)
   apply (rule hide_tr_prefix_if)
   apply (simp)

   apply (simp add: rest_tr_def)
   apply (rule hide_tr_prefix_if)
   apply (simp)
done

lemma in_traces_Pipe_sound:
  "[| inj left; inj mid; inj right; ALL x y. right x ~= mid y;
       ALL x y. left x ~= mid y; ALL x y. right x ~= left y;
       EX s t.
          s :t traces P M &
          t :t traces Q M &
          sett s <= insert Tick (Ev ` (range left Un range right)) &
          sett t <= insert Tick (Ev ` (range left Un range right)) &
          sett u <= insert Tick (Ev ` (range left Un range right)) &
             u rest-tr range left = s rest-tr range left &
            (s rest-tr range right) [[right<==>left]]* (t rest-tr range left) &
             t rest-tr range right = u rest-tr range right &
          pipe_condition_prefix u s t left right |]
    ==> u :t traces (P <[left,mid,right]> Q) M"
 apply (simp add: Pipe_def)
 apply (simp add: in_traces_Alpha_parallel in_traces)
 apply (elim conjE exE)
 apply (insert pipe_condition_prefix_comp[of left mid right])
 apply (drule_tac x="u" in spec)
 apply (drule_tac x="s" in spec)
 apply (drule_tac x="t" in spec)
 apply (auto)
done

(* ------------------------------------------------ *
       (main) the semantic clause of Pipe (T)
 * ------------------------------------------------ *)

lemma in_traces_Pipe: 
  "[| inj left; inj mid; inj right ;
      ALL x y. right x ~= mid y ;
      ALL x y. left x ~= mid y ;
      ALL x y. right x ~= left y |] ==>
    (u :t traces (P <[left,mid,right]> Q) M) =
    (EX s t. s :t traces P M &
             t :t traces Q M &
                 sett s <= insert Tick (Ev ` (range left Un range right)) &
                 sett t <= insert Tick (Ev ` (range left Un range right)) &
                 sett u <= insert Tick (Ev ` (range left Un range right)) &
             u rest-tr range left = s rest-tr range left &
            (s rest-tr range right) [[right<==>left]]* (t rest-tr range left) &
             t rest-tr range right = u rest-tr range right &
         pipe_condition_prefix u s t left right)"
apply (rule iffI)

(* => *)
 apply (simp add: Pipe_def)
 apply (simp add: in_traces_Alpha_parallel in_traces)
 apply (elim conjE exE)
 apply (rename_tac v s t)
 apply (rule_tac x="s" in exI, simp)
 apply (rule_tac x="t" in exI, simp)
 apply (simp add: in_traces_Pipe_comp_sub)

(* <= *)
 apply (rule in_traces_Pipe_sound)
 apply (auto)
done

(* ====================== failures ====================== *)

lemma in_failures_Pipe_refusal_set_comp1:
    "[| ALL x y. right x ~= mid y ;
          ALL x y. left x ~= mid y ;
          ALL x y. right x ~= left y ;
          (Ev ` range mid Un Z) Int
          insert Tick (Ev ` (range left Un range mid Un (range mid Un range right))) =
          X Int insert Tick (Ev ` (range left Un range mid)) Un
          Y Int insert Tick (Ev ` (range mid Un range right)) |]
       ==> Z Int (Ev ` range left) = X Int (Ev ` range left)"
apply (elim add_not_eq_symE)
apply (rule equalityI)
apply (rule subsetI)
apply (erule equalityE)
apply (rotate_tac -2)
apply (erule subsetE)
apply (drule_tac x="x" in bspec)
apply (force)
apply (force)

apply (rule subsetI)
apply (erule equalityE)
apply (rotate_tac -1)
apply (erule subsetE)
apply (drule_tac x="x" in bspec)
apply (force)
apply (force)
done

lemma in_failures_Pipe_refusal_set_comp2:
    "[| ALL x y. right x ~= mid y ;
          ALL x y. left x ~= mid y ;
          ALL x y. right x ~= left y ;
          (Ev ` range mid Un Z) Int
          insert Tick (Ev ` (range left Un range mid Un (range mid Un range right))) =
          X Int insert Tick (Ev ` (range left Un range mid)) Un
          Y Int insert Tick (Ev ` (range mid Un range right)) |]
       ==> Y Int Ev ` range right = Z Int Ev ` range right"
apply (elim add_not_eq_symE)
apply (rule equalityI)
apply (rule subsetI)
apply (erule equalityE)
apply (rotate_tac -1)
apply (erule subsetE)
apply (drule_tac x="x" in bspec)
apply (force)
apply (force)

apply (rule subsetI)
apply (erule equalityE)
apply (rotate_tac -2)
apply (erule subsetE)
apply (drule_tac x="x" in bspec)
apply (force)
apply (force)
done

lemma in_failures_Pipe_refusal_set_comp3:
    "[| ALL x y. right x ~= mid y ;
          ALL x y. left x ~= mid y ;
          ALL x y. right x ~= left y ;
          (Ev ` range mid Un Z) Int
          insert Tick (Ev ` (range left Un range mid Un (range mid Un range right))) =
          X Int insert Tick (Ev ` (range left Un range mid)) Un
          Y Int insert Tick (Ev ` (range mid Un range right)) |]
       ==> Ev ` range mid <= X Un Y"
apply (elim add_not_eq_symE)
apply (rule subsetI)
apply (erule equalityE)
apply (rotate_tac -2)
apply (erule subsetE)
apply (drule_tac x="x" in bspec)
apply (force)
apply (force)
done

lemma in_failures_Pipe_refusal_set_comp4:
    "[| ALL x y. right x ~= mid y ;
          ALL x y. left x ~= mid y ;
          ALL x y. right x ~= left y ;
          (Ev ` range mid Un Z) Int
          insert Tick (Ev ` (range left Un range mid Un (range mid Un range right))) =
          X Int insert Tick (Ev ` (range left Un range mid)) Un
          Y Int insert Tick (Ev ` (range mid Un range right)) |]
       ==> Z Int {Tick} = (X Un Y) Int {Tick}"
apply (elim add_not_eq_symE)
apply (rule equalityI)
apply (rule subsetI)
apply (erule equalityE)
apply (rotate_tac -2)
apply (erule subsetE)
apply (drule_tac x="Tick" in bspec)
apply (force)
apply (force)

apply (rule subsetI)
apply (erule equalityE)
apply (rotate_tac -1)
apply (erule subsetE)
apply (drule_tac x="Tick" in bspec)
apply (force)
apply (force)
done

(* ------------------------------------------------ *
       the semantic clause (soundness, set <= )
 * ------------------------------------------------ *)

lemma in_failures_Pipe_refusal_set_sound1:
   "[| ALL x y. right x ~= mid y; 
       ALL x y. left x ~= mid y; 
       ALL x y. right x ~= left y;
          Z Int Ev ` range left = X Int Ev ` range left;
          Y Int Ev ` range right = Z Int Ev ` range right;
          Ev ` range mid <= X Un Y;
          Z Int {Tick} = (X Un Y) Int {Tick} |]
       ==> (Ev ` range mid Un Z) Int
           insert Tick (Ev ` (range left Un range mid Un (range mid Un range right))) <=
           X Int insert Tick (Ev ` (range left Un range mid)) Un
           Y Int insert Tick (Ev ` (range mid Un range right))"
apply (rule subsetI)
apply (simp)
apply (elim disjE conjE exE)
apply (simp_all)

(* 1 *)
apply (simp add: image_iff)

(* 2 *)
apply (erule subsetE)
apply (drule_tac x="x" in bspec, simp)
apply (simp)
apply (force)

(* 3 *)
apply (rotate_tac -3)
apply (erule equalityE)
apply (rotate_tac -2)
apply (erule subsetE)
apply (drule_tac x="Tick" in bspec, simp)
apply (simp)

(* 4 *)
apply (simp add: image_iff)
apply (elim conjE bexE)
apply (simp)
apply (elim disjE conjE bexE)
apply (simp_all)

 (* 4.1 *)
 apply (rotate_tac 3)
 apply (erule equalityE)
 apply (rotate_tac -2)
 apply (erule subsetE)
 apply (drule_tac x="Ev xa" in bspec, simp)
 apply (simp)

 (* 4.2 *)
 apply (erule subsetE)
 apply (drule_tac x="x" in bspec, simp)
 apply (simp)

 (* 4.3 *)
 apply (rotate_tac 4)
 apply (erule equalityE)
 apply (rotate_tac -1)
 apply (erule subsetE)
 apply (drule_tac x="Ev xa" in bspec, simp)
 apply (simp)
done

(* ------------------------------------------------ *
       the semantic clause (soundness, set => )
 * ------------------------------------------------ *)

lemma in_failures_Pipe_refusal_set_sound2:
   "[| ALL x y. right x ~= mid y; 
       ALL x y. left x ~= mid y; 
       ALL x y. right x ~= left y;
          Z Int Ev ` range left = X Int Ev ` range left;
          Y Int Ev ` range right = Z Int Ev ` range right;
          Ev ` range mid <= X Un Y;
          Z Int {Tick} = (X Un Y) Int {Tick} |]
       ==> 
           X Int insert Tick (Ev ` (range left Un range mid)) Un
           Y Int insert Tick (Ev ` (range mid Un range right)) <=
           (Ev ` range mid Un Z) Int
           insert Tick (Ev ` (range left Un range mid Un (range mid Un range right)))"
apply (rule subsetI)
apply (simp)
apply (elim disjE conjE exE)
apply (simp_all)

 (* 1 *)
 apply (rotate_tac -3)
 apply (erule equalityE)
 apply (rotate_tac -1)
 apply (erule subsetE)
 apply (drule_tac x="Tick" in bspec, simp)
 apply (simp)

 (* 2 *)
 apply (simp add: image_iff)
 apply (elim conjE bexE)
 apply (simp)
 apply (elim disjE conjE bexE)

  (* 2.1 *)
  apply (rotate_tac 3)
  apply (erule equalityE)
  apply (rotate_tac -1)
  apply (erule subsetE)
  apply (drule_tac x="Ev xa" in bspec, simp)
  apply (simp)

  (* 2.2 *)
  apply (force)

 (* 3 *)
 apply (rotate_tac 6)
 apply (erule equalityE)
 apply (rotate_tac -1)
 apply (erule subsetE)
 apply (drule_tac x="Tick" in bspec, simp)
 apply (simp)

 (* 4 *)
 apply (simp add: image_iff)
 apply (elim disjE conjE bexE)
 apply (simp)
 apply (elim disjE conjE bexE)
 apply (force)

 apply (rotate_tac 4)
 apply (erule equalityE)
 apply (rotate_tac -2)
 apply (erule subsetE)
 apply (drule_tac x="Ev xa" in bspec, simp)
 apply (simp)
done

(* --- set lemma --- *)

lemma in_failures_Pipe_refusal_set_sound:
   "[| ALL x y. right x ~= mid y; 
       ALL x y. left x ~= mid y; 
       ALL x y. right x ~= left y;
          Z Int Ev ` range left = X Int Ev ` range left;
          Y Int Ev ` range right = Z Int Ev ` range right;
          Ev ` range mid <= X Un Y;
          Z Int {Tick} = (X Un Y) Int {Tick} |]
       ==> (Ev ` range mid Un Z) Int
           insert Tick (Ev ` (range left Un range mid Un (range mid Un range right))) =
           X Int insert Tick (Ev ` (range left Un range mid)) Un
           Y Int insert Tick (Ev ` (range mid Un range right))"
apply (rule equalityI)
apply (simp add: in_failures_Pipe_refusal_set_sound1)
apply (rule in_failures_Pipe_refusal_set_sound2)
apply (simp_all)
done


(* ------------------------------------------------ *
       (main) the semantic clause of Pipe (F)
 * ------------------------------------------------ *)

lemma in_failures_Pipe: 
  "[| inj left; inj mid; inj right;
      ALL x y. right x ~= mid y ;
      ALL x y. left x ~= mid y ;
      ALL x y. right x ~= left y |] ==>
    (f :f failures (P <[left,mid,right]> Q) M) =
   (EX u Z.
     f = (u, Z) &
     (EX s t X Y.
         (s, X) :f failures P M &
         (t, Y) :f failures Q M &
         Z Int (Ev ` (range left)) = X Int (Ev ` (range left)) &
         Y Int (Ev ` (range right)) = Z Int (Ev ` (range right)) &
         Ev ` range mid <= [[right<==>mid]]inv X Un [[left<==>mid]]inv Y &
         Z Int {Tick} = (X Un Y) Int {Tick} &
                 sett s <= insert Tick (Ev ` (range left Un range right)) &
                 sett t <= insert Tick (Ev ` (range left Un range right)) &
                 sett u <= insert Tick (Ev ` (range left Un range right)) &
             u rest-tr range left = s rest-tr range left &
            (s rest-tr range right) [[right<==>left]]* (t rest-tr range left) &
             t rest-tr range right = u rest-tr range right &
         pipe_condition_prefix u s t left right))"
apply (rule iffI)

(* => *)
 apply (simp add: Pipe_def)
 apply (simp add: in_failures_Alpha_parallel in_failures)
 apply (elim conjE exE)
 apply (rename_tac u Z X Y s t)
 apply (rule_tac x="u --tr range mid" in exI)
 apply (rule_tac x="Z" in exI)
 apply (simp)
 apply (rule_tac x="s" in exI)
 apply (rule_tac x="t" in exI)
 apply (rule_tac x="[[right<==>mid]]inv X" in exI, simp)
 apply (rule_tac x="[[left<==>mid]]inv Y" in exI, simp)
 apply (elim add_not_eq_symE)
 apply (elim conjE exE)
  apply (simp add: Renaming_channel_ren_inv_Int_eq)
  apply (simp add: in_failures_Pipe_refusal_set_comp1)
  apply (simp add: in_failures_Pipe_refusal_set_comp2)
  apply (simp add: Renaming_channel_ren_inv_ren_inv_eq)
  apply (simp add: in_failures_Pipe_refusal_set_comp3)
  apply (simp add: Int_Un_distrib2 Renaming_channel_ren_inv_Int_Tick_eq
                   in_failures_Pipe_refusal_set_comp4)
 apply (simp add: in_traces_Pipe_comp_sub)

(* <= *)
 apply (simp add: Pipe_def)
 apply (simp add: in_failures_Alpha_parallel in_failures)
 apply (elim conjE exE)
 apply (insert pipe_condition_prefix_comp[of left mid right])
 apply (drule_tac x="u" in spec)
 apply (drule_tac x="s" in spec)
 apply (drule_tac x="t" in spec)
 apply (simp)
 apply (elim conjE exE)
 apply (elim add_not_eq_symE)
 apply (simp)

 apply (rule_tac x="v" in exI, simp)
  apply (rule_tac x="[[right<==>mid]]inv X" in exI)
  apply (rule_tac x="[[left<==>mid]]inv Y" in exI)
  apply (simp add: Renaming_channel_ren_inv_ren_inv_eq)
  apply (subgoal_tac
   "Z Int Ev ` range left = [[right<==>mid]]inv X Int Ev ` range left")
  apply (subgoal_tac
   "[[left<==>mid]]inv Y Int Ev ` range right = Z Int Ev ` range right")
  apply (simp)

  apply (rule)
  apply (rule in_failures_Pipe_refusal_set_sound)
  apply (simp_all)
  apply (simp add: Int_Un_distrib2 Renaming_channel_ren_inv_Int_Tick_eq)

  apply (rule)
  apply (rule_tac x="s" in exI, simp)
  apply (rule_tac x="t" in exI, simp)
  apply (simp add: Renaming_channel_ren_inv_Int_eq)
  apply (simp add: Renaming_channel_ren_inv_Int_eq)
done

end
